package net.sergeych.parsec3

import kotlinx.coroutines.flow.Flow
import net.sergeych.mp_tools.globalLaunch

class AdapterBuilder<S, H : CommandHost<S>>(
    val api: H,
    val exceptionRegistry: ExceptionsRegistry = ExceptionsRegistry(),
    f: AdapterBuilder<S, H>.() -> Unit,
) {

    internal var sessionProducer: (suspend () -> S) = { Unit as S}
        private set


    fun newSession(f: suspend () -> S) {
        sessionProducer = f
    }

    /**
     * Register command implementation
     */
    fun <A, R> on(ca: CommandDescriptor<A, R>, block: suspend S.(A) -> R) {
        api.on(ca, block)
    }

    inline fun <reified T : Throwable> addError(code: String, noinline handler: (String?) -> T) {
        exceptionRegistry.register(code, handler)
    }

    suspend fun createWith(input: Flow<ByteArray>, f: suspend (ByteArray)->Unit ): Adapter<S> {
        return Adapter<S>(sessionProducer(), api, exceptionRegistry) { f(it) }
            .also { a-> globalLaunch { input.collect { a.receiveFrame(it)} } }
    }

    suspend fun create(f: suspend (ByteArray) -> Unit): Adapter<S> {
        return Adapter(sessionProducer(), api, exceptionRegistry, f)
    }

    init {
        f(this)
    }

}