package net.sergeych.parsec3

import io.ktor.client.*
import io.ktor.client.plugins.websocket.*
import io.ktor.websocket.*
import kotlinx.coroutines.CompletableDeferred
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.launch
import net.sergeych.mp_logger.LogTag
import net.sergeych.mp_logger.info
import net.sergeych.mp_tools.globalLaunch

class Parsec3WSClient<S, H : CommandHost<S>>(
    val url: String,
    val api: H,
    val exceptionsRegistry: ExceptionsRegistry = ExceptionsRegistry(),
    f: AdapterBuilder<S, H>.() -> Unit,
) : LogTag("P3WSC"), Parsec3Transport<S> {

    val builder = AdapterBuilder(api, exceptionsRegistry, f)

    private val _connectionFlow = MutableStateFlow(false)
    private val closeFlow = MutableStateFlow(false)
    private val reconnectFlow = MutableStateFlow(false)
    override val connectedFlow: StateFlow<Boolean> = _connectionFlow


    init {
        start()
    }

    override fun close() {
        if( closeFlow.value == false ) closeFlow.value = true
    }

    override fun reconnect() {
        reconnectFlow.value = true
    }

    var deferredAdapter = CompletableDeferred<Adapter<S>>()
    private set

    override suspend fun adapter(): Adapter<S> = deferredAdapter.await()

    fun start() {
        globalLaunch {
            while(closeFlow.value != true) {
                reconnectFlow.value = false
                client.webSocket(url) {
                    info { "Connected to $url" }
                    val a = builder.create { send(Frame.Binary(true, it)) }
                    _connectionFlow.value = true
                    launch { closeFlow.collect { if (it == true) close() } }
                    launch { reconnectFlow.collect { if (it == true) close() } }
                    deferredAdapter.complete(a)
                    for (f in incoming)
                        if (f is Frame.Binary) a.receiveFrame(f.data)
                    // when we leave connection will be closed. So far we do not close the adapter,
                    // should we?
                    _connectionFlow.value = false
                    deferredAdapter = CompletableDeferred()
                }
            }
        }
    }

    companion object {
        private val client = HttpClient {
            install(WebSockets) {
                // Configure WebSockets
            }
        }
    }
}
